library(readr)
library(ggplot2)
library(ineq)
library(boot)
library(reshape2)
library(ggrepel)
library(RColorBrewer)
library(Hmisc)
library(cowplot)
theme_set(theme_cowplot())

baseDir <- "researchBias/inputData/"
outputDir <- "researchBias/"
#Read in the Entrez human gene data
humanGenes <- read_delim(paste(baseDir, "human_genes.txt", sep=""), delim="\t", col_names =T)

##################
# coreFunctions
##################
giniFunctionBoot <- function(originalData, indices, type) {
	return(ineq(originalData[indices], type=type))
}

versionTimes <- readr::read_tsv(paste(baseDir, "GO_annCounts/versionTimes.tsv", sep=""), col_names = F)
versionTimes$X1 <- paste(versionTimes$X1, ".tsv", sep="")
giniOrganismFunction <- function(directory, type="Gini") {
	curFrame <- data.frame()
	for(curFile in list.files(directory, full.names = F)) {
		goData <- read_tsv(paste(directory, curFile, sep=""))
		dirSplit <- strsplit(directory,"/")[[1]]
		ineqData <- goData$Freq
		curFrame <- rbind(curFrame, data.frame(value = ineq(ineqData, type=type), metric = type, organism=dirSplit[length(dirSplit)], 
																					 filename=paste(directory, curFile, sep=""), 
																					 timestamp= versionTimes[versionTimes$X1==curFile,]$X2))
	}
	return(curFrame)
}

giniOrganismFunctionCI <- function(directory, type="Gini") {
	curFrame <- data.frame()
	for(curFile in list.files(directory, full.names = F)) {
		goData <- read_tsv(paste(directory, curFile, sep=""))
		dirSplit <- strsplit(directory,"/")[[1]]
		ineqData <- goData$Freq
		bootRes <- boot.ci(boot(goData$Freq, giniFunctionBoot, 1000, sim="balanced", type=type), type="norm")$normal
		curFrame <- rbind(curFrame, data.frame(percentCI = bootRes[1,1], CILower = bootRes[1,2], CIUpper = bootRes[1,3], 
																					 value=ineq(goData$Freq, type=type), metric = type, organism=dirSplit[length(dirSplit)], 
																					 filename=paste(directory, curFile, sep=""), 
																					 timestamp= versionTimes[versionTimes$X1==curFile,]$X2))
	}
	return(curFrame)
}

##################
# Analysis in GO data
##################
giniAlgorithm <- data.frame()
for(algorithm in c("Gini", "RS", "Atkinson", "Theil", "Kolm","square.var","entropy", "var")) {
	orgData <- giniOrganismFunction(paste(baseDir, "GO_annCounts/annCountsNonIEAND/human/",sep=""), type=algorithm)
	orgData$value <- orgData$value/orgData$value[1]
	giniAlgorithm <- rbind(giniAlgorithm,orgData )
}
giniAlgorithm$date <- as.POSIXct(giniAlgorithm$timestamp, origin = "1970-01-01")
levels(giniAlgorithm$metric) <- c("Gini coefficient", "Ricci-Schutz coefficient", "Atkinson's measure", "Theil's entropy",
																	"Kolm's measure","Squared variation","Generalized entropy", "Variation")


 # ciData <- giniOrganismFunctionCI(paste(baseDir, "GO_annCounts/annCountsNonIEAND/human/",sep=""), type="Gini")
 # save(ciData, file=paste(baseDir, "RData/ciData.RData", sep=""))
load(paste(baseDir, "RData/ciData.RData", sep=""))
ciData$date <- as.POSIXct(ciData$timestamp, origin = "1970-01-01")

pdf(paste(outputDir, "giniPlot.pdf", sep=""))
	confidenceIntPlot <- ggplot(ciData, aes(x=date, y=value)) + 
		geom_ribbon(aes(ymin=CILower,ymax=CIUpper), lwd=2) + 
		scale_y_continuous(limits = c(0,1)) + xlab("Year") + ylab("Gini Coefficient") + guides(color=FALSE) + 
		theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
					legend.justification = c(0,1), legend.background = element_rect(fill=F)) 
	confidenceIntPlot
	
	algorithmPlot <- ggplot(giniAlgorithm, aes(x=date, y=value, color=metric)) + geom_line(lwd=1) + xlab("Year") + 
		ylab("Inequality Value (Normalized to 2001)")+ 
		theme(text = element_text(size=20), axis.title=element_text(size=24), legend.position = c(0.05,0.95), 
					legend.justification = c(0,1), legend.background = element_rect(fill=F), legend.key.size = unit(2, 'lines') ) + 
		guides(color=guide_legend(title="Inequality Metric"))
	algorithmPlot
dev.off()

##############
# Analysis in only high-throughput assay
##############
highThroughput <- giniOrganismFunction(paste(baseDir, "GO_annCounts/annCountsIEP/human/",sep=""), type="Gini")
highThroughput$date <- as.POSIXct(highThroughput$timestamp, origin = "1970-01-01")
ggplot(highThroughput, aes(x=date, y=value)) + geom_line(lwd=1) + xlab("Year") + 
	ylab("Gini Inequality")

##################
# Simulate human data according to different growth models
##################

humanSimulate <- ciData
humanSimulate$ineq <- ciData$value
humanSimulate$type <- "Gini"
humanSimulate <- humanSimulate[order(humanSimulate$timestamp),]
humanSimulate$iteration <- 0:(nrow(humanSimulate)-1)
humanSimulate$model <- "Actual Data"

initialState <- read_tsv(paste(baseDir, "GO_annCounts/annCountsNonIEAND/human/gene_association.goa_human.1.tsv", sep=""))
finalState <- read_tsv(paste(baseDir, "GO_annCounts/annCountsNonIEAND/human/goa_human.gaf.172.tsv", sep=""))
numVersions <- nrow(humanSimulate) - 1
annotationsPerVersion <- round((sum(finalState$Freq) - sum(initialState$Freq)) / numVersions)
equalWeightModel <- function(curState, numAnnotations, ...) {
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations/nrow(newState)
	return(newState)
}
initialWeightModel <- function(curState, numAnnotations, initialState) {
	initialProb <- initialState$Freq/sum(initialState$Freq)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * initialProb
	return(newState)
}
initialWeightRandomFocusModel <- function(curState,numAnnotations,  initialState) {
	initialProb <- initialState$Freq/sum(initialState$Freq)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * sample(initialProb)
	return(newState)
}
squaredInitialWeightModel <- function(curState,numAnnotations,  initialState) {
	initialProb <- initialState$Freq ** 2
	initialProb <- initialProb * 1/sum(initialProb)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * initialProb
	return(newState)
}
cubicInitialWeightModel <- function(curState,numAnnotations, initialState) {
	initialProb <- initialState$Freq ** 3
	initialProb <- initialProb * 1/sum(initialProb)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * initialProb
	return(newState)
}
expInitialWeightModel <- function(curState, numAnnotations, initialState) {
	initialProb <- exp(initialState$Freq)
	initialProb <- initialProb * 1/sum(initialProb)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * initialProb
	return(newState)
}
squaredWeightModel <- function(curState,numAnnotations, initialState) {
	curProb <- curState$Freq ** 2
	curProb <- curProb * 1/sum(curProb)
	newState <- curState
	newState$Freq <- newState$Freq + numAnnotations * curProb
	return(newState)
}
simulateModel <- function(initialState, modelFunction, modelName, type="Gini") {
	curState <- initialState
	numAnnotations <- sum(initialState$Freq)
	ineqFrame <- data.frame(ineq=ineq(curState$Freq, type=type), type=type, model=modelName, date=humanSimulate[1,"date"])
	for(i in 2:nrow(humanSimulate)) {
		prevAnnotations <- numAnnotations
		numAnnotations <- sum(read_tsv(as.character(humanSimulate[i,"filename"]))$Freq)
		annotChange <- numAnnotations - prevAnnotations
		if(annotChange < 0) { annotChange <- 0}
		curState <- modelFunction(curState, numAnnotations=annotChange, initialState=initialState)
		ineqFrame <- rbind(ineqFrame, data.frame(ineq=ineq(curState$Freq, type=type), type=type, 
																						 model=modelName, date=humanSimulate[i,"date"]) )
		
	}
	return(ineqFrame)
}
modelResults <- simulateModel(initialState=initialState, modelFunction = equalWeightModel, modelName = "Equal Weight")
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = initialWeightRandomFocusModel, 
																		modelName = "Initial Weight Random Focus"))
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = initialWeightModel, 
																modelName = "Initial Weight"))
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = squaredInitialWeightModel, 
																		modelName = "Squared Initial Weight"))
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = cubicInitialWeightModel, 
																		modelName = "Cubic Initial Weight"))
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = squaredWeightModel, 
																		modelName = "Squared Weight"))
modelResults <- rbind(modelResults, simulateModel(initialState=initialState, modelFunction = expInitialWeightModel, 
																		modelName = "Exponential Initial Weight"))
modelResults <- rbind(modelResults, humanSimulate[,colnames(modelResults)])
modelResults$model <- factor(modelResults$model, levels=unique(modelResults$model)[length(unique(modelResults$model)):1])

pdf(paste(outputDir, "modelResults.pdf", sep=""))
modelPlot <- ggplot(modelResults, aes(x=date, y=ineq, color=model)) + 
	geom_label_repel(data=modelResults[modelResults$date==max(modelResults$date),], aes(label=model), hjust=1, vjust=0, size=8)+
	geom_line(lwd=1, lty=6) + geom_line(data=humanSimulate, lwd=2, color="black") +  scale_y_continuous(limits = c(0,1)) +
	ylab("Gini Coefficient") + xlab("Year") + guides(color=F) +
	theme(text = element_text(size=20), axis.title=element_text(size=24))
modelPlot
dev.off()

##################
# Analysis in other organisms
##################
# ftp://ftp.ebi.ac.uk/pub/databases/GO/goa/old
allOrganismsNonIEAND <- data.frame()
annDir <- paste(baseDir, "GO_annCounts/annCountsNonIEAND/", sep = "")
for(curDir in list.dirs(annDir, recursive=F)) {
	allOrganismsNonIEAND <- rbind(allOrganismsNonIEAND, giniOrganismFunction(paste(curDir,"/",sep=""), type="Gini"))
}
allOrganismsNonIEAND$goData <- "Non-IEA, Non-ND, Non-NR"

allOrganismsMostRecent <- data.frame() 
for(curOrganism in unique(allOrganismsNonIEAND$organism)) {
	curSubset <- allOrganismsNonIEAND[allOrganismsNonIEAND$organism==curOrganism,]
	curSubset <- curSubset[curSubset$timestamp==max(curSubset$timestamp),]
	allOrganismsMostRecent <- rbind(allOrganismsMostRecent, curSubset)
}

allOrganismsNonIEAND$date <- as.POSIXct(allOrganismsNonIEAND$timestamp, origin = "1970-01-01")
allOrganismsMostRecent$date <- as.POSIXct(allOrganismsMostRecent$timestamp, origin = "1970-01-01")

allOrganismsNonIEAND$organism <- factor(as.character(allOrganismsNonIEAND$organism), levels = as.character(allOrganismsMostRecent$organism[order(allOrganismsMostRecent$value, decreasing = T)]))
levels(allOrganismsNonIEAND$organism) <- capitalize(levels(allOrganismsNonIEAND$organism))

##################
# Examine different data subsets
##################
dataSubsetFrame <- giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsCC/human/", type="Gini")
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsBP/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsMF/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsExpEvidence/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsCompAnalysis/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsAuthState/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsCurateState/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsAutomaticAssign/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCounts/human/", type="Gini"))
dataSubsetFrame <- rbind(dataSubsetFrame, giniOrganismFunction("researchBias/inputData/GO_annCounts/annCountsNonIEAND/human/", type="Gini"))
dataSubsetFrame$subset <- factor(c(rep("Cellular Component Only", 170), rep("Biological Process Only", 170), 
														rep("Molecular Function Only", 170), rep("Experimental Evidence Codes", 170),
														rep("Computational Analysis Evidence Codes", 170), rep("Author Statement Evidence Codes", 170), 
														rep("Curatorial Statement Evidence Codes", 170),rep("Automatically-Assigned Evidence Codes", 170),  
														rep("All GO Data", 170), rep("Non-IEA,ND,NR", 170)), levels= c(
																	"Experimental Evidence Codes", "Computational Analysis Evidence Codes", "Author Statement Evidence Codes",
																	"Curatorial Statement Evidence Codes", "Automatically-Assigned Evidence Codes",
																	"Cellular Component Only", "Biological Process Only", "Molecular Function Only","All GO Data", "Non-IEA,ND,NR" ))
dataSubsetFrame$date <- as.POSIXct(dataSubsetFrame$timestamp, origin = "1970-01-01")

pdf(paste(outputDir, "giniPlotOrganisms.pdf", sep=""), width=10, height=10)
organismPlot <- ggplot(allOrganismsNonIEAND, aes(x=date, y=value, color=organism)) + geom_line(lwd=1) + 
	xlab("Year") + ylab("Gini Coefficient")+ scale_y_continuous(limits = c(0,1)) +  
	theme(text = element_text(size=20), axis.title=element_text(size=24), legend.position = c(0.01,1.05), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F), legend.key.size = unit(1.5, 'lines')) + 
	guides(color=guide_legend(title="Organism"))
organismPlot

goDataSubsetPlot <- ggplot(dataSubsetFrame, aes(x=date, y=value, color=subset)) +
	geom_line(lwd=1) + scale_y_continuous(limits = c(0,1)) + xlab("Year") + ylab("Gini Coefficient") + 
	theme(text = element_text(size=20), axis.title=element_text(size=24), legend.position = c(0.0,1), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F), legend.key.size = unit(1.5, 'lines')) + 
	guides(color=guide_legend(title="GO Data Subset"))
goDataSubsetPlot
dev.off()

##################
# Analysis in other databases
##################
#################
# Pubpular
#################
#Read in the pubpular data
pubpularDir <- paste(baseDir, "pubpular/", sep="")
if(exists("pubpularFrame")) {
	rm(pubpularFrame)
	rm(pubpularIneqFrame)
}
for(filename in list.files(pubpularDir, pattern="*.csv")) {
	
	diseaseName <- strsplit(filename, "\\.")[[1]][[1]]
	tempPub <- read.csv(paste(pubpularDir, filename,sep=""))
	tempPub <- tempPub[,c("GN", "Term_count")]
	tempIneq <- data.frame(diseaseName=diseaseName, gini=ineq(tempPub$Term_count))
	if(exists("pubpularFrame")) {
		pubpularFrame <- rbind(pubpularFrame, tempPub)
		pubpularIneqFrame <- rbind(pubpularIneqFrame, tempIneq)
	} else {
		pubpularFrame <- tempPub
		pubpularIneqFrame <- tempIneq
	}
}
pubpularFrame$placeholder <- rep("Fake",nrow(pubpularFrame))
pubpularSummarized <- dcast(pubpularFrame,  GN ~ placeholder, fun.aggregate = sum, value.var="Term_count")

#################
# GeneRIF
#################
#Read in the gene RIF data
geneRif <- readr::read_delim(paste(baseDir, "generifs_human",sep=""), delim="\t")
colnames(geneRif)[1:2] <-  c("tax_id","GeneID")
geneRifGenes <- merge(geneRif, humanGenes, by="GeneID")
geneRifTable <- as.data.frame(table(geneRifGenes$Symbol))

#################
# Reactome
#################
#Read in the Reactome data
reactome <- readr::read_delim(paste(baseDir, "reactome2Counts.tsv",sep=""), delim="\t")
write.csv(reactome[,c("uniprot")], file=paste(baseDir,"reactomeUniprots.tsv", sep=""), quote=F)

#Map uniprot to gene names
uniprotToGene <- read_delim(paste(baseDir, "uniprot2geneName.tab",sep=""), delim="\t")
uniprotToGene$To <- toupper(uniprotToGene$To)
geneMap <- function(uniprot){
	curRow <- uniprotToGene$To[uniprotToGene$From==uniprot]
	if(length(curRow)==0) { return(NA) }
	if(length(curRow)==1) { return(curRow) }
	return(curRow[[1]])
}
reactome$geneName <- sapply(reactome$uniprot, geneMap)
reactome <- reactome[!duplicated(reactome$geneName),]
reactome <- reactome[!is.na(reactome$geneName),]

#################
# CTD
#################
#Read in the CTD data
ctdChemical <- readr::read_delim(paste(baseDir, "CTD_chem_gene_ixns.tsv",sep=""), delim="\t")
ctdChemicalTable <- as.data.frame(table(ctdChemical$GeneSymbol))

ctdPathway <- readr::read_delim(paste(baseDir, "CTD_genes_pathways.tsv",sep=""), delim="\t")
ctdPathwayTable <- as.data.frame(table(ctdPathway$GeneSymbol))

#################
# DrugBank
#################
#Read in the drugbank
drugbank <- readr::read_delim(paste(baseDir, "human_genesdrugs.tsv",sep=""), delim="\t", col_names = F)
colnames(drugbank) <- c("drugName", "drugStatus","drugEffect","geneName")
drugbankTable <- as.data.frame(table(drugbank$geneName))

#################
# PDB
#################
#Read in the PDB
pdb <- readr::read_delim(paste(baseDir, "pdbResults.tsv",sep=""), delim=",", col_names =T)
pdbUnique <- pdb[!duplicated(pdb$`PDB ID`),]
splitFun <- function(curStr) {
	return(strsplit(curStr, ",")[[1]])
}
pdbGenes <- unlist(sapply(pdbUnique$`Gene Name`, splitFun))
pdbGeneTable <- as.data.frame(table(pdbGenes))


#################
# Merge data across analyses
#################
#Look at patterns across genes
drugbankMerge <- drugbankTable
colnames(drugbankMerge) <- c("geneName","drugBank")
ctdChemicalMerge <- ctdChemicalTable
colnames(ctdChemicalMerge) <- c("geneName","ctdChemical")
ctdPathwayMerge <- ctdPathwayTable
colnames(ctdPathwayMerge) <- c("geneName","ctdPathway")
pubpularMerge <- pubpularSummarized
colnames(pubpularMerge) <- c("geneName","pubpular")
pdbMerge <- pdbGeneTable
colnames(pdbMerge) <- c("geneName","pdb")
reactomeMerge <- reactome[,c("geneName","pathways")]
colnames(reactomeMerge) <- c("geneName", "reactome")
geneRifMerge <- geneRifTable
colnames(geneRifMerge) <- c("geneName", "geneRif")

mergedByGene <- merge(
		merge(merge(merge(merge(merge(pubpularMerge, ctdPathwayMerge, by="geneName", all = T), 
														ctdChemicalMerge, by="geneName", all = T), drugbankMerge, by="geneName", all = T), 
														pdbMerge, by="geneName", all = T), 
														reactomeMerge, by="geneName", all = T),
														geneRifMerge, by="geneName", all = T)
rownames(mergedByGene) <- mergedByGene$geneName
mergedByGene <- mergedByGene[,2:ncol(mergedByGene)]

mergedHuman <- mergedByGene[rownames(mergedByGene) %in% humanGenes$Symbol,]

#################
# Calculate gini coefficients for all datasets
#################
singleIneqFrame <- data.frame(dataset=c("GeneRIF", "Reactome", rep("CTD",2), "DrugBank","PDB",rep("PubPular",2),"Global"), 
												fullName=c("GeneRIF", "Reactome", "CTD Chemical-Gene", "CTD Pathway", "DrugBank","PDB",
																	 "Pubpular Total", "Pubpular Median","Global Annotation Inequality"),
												gini=c(ineq(mergedHuman$geneRif),ineq(mergedHuman$reactome),ineq(mergedHuman$ctdChemical),
															 ineq(mergedHuman$ctdPathway), ineq(mergedHuman$drugBank),
															 ineq(mergedHuman$pdb),ineq(mergedHuman$pubpular),median(pubpularIneqFrame$gini), ineq(rowSums(mergedHuman))))


mypalette <- brewer.pal(8, "Set1")
pdf(paste(outputDir, "giniPlotEverything.pdf", sep=""))
everythingPlot <- ggplot(ciData, aes(x=date, y=value)) + 
	geom_point(data=singleIneqFrame, aes(y=gini, color=dataset), x=as.POSIXct(strptime("2017","%Y")), size=2) +
	geom_label(data=singleIneqFrame, aes(y=gini, label=fullName, color=dataset), x=as.POSIXct(strptime("2017","%Y")), box.padding = unit(0.3, 'lines'), size=8, hjust=1)+
	annotate("label", x=as.POSIXct(strptime("2011","%Y")), y=0.53, label="Gene Ontology Inequality", size=8) +
	geom_line(lwd=2) + 
	scale_y_continuous(limits = c(0,1)) + xlab("Year") + ylab("Gini Coefficient") + guides(color=FALSE) + 
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F)) 
everythingPlot
dev.off()

##############
# Assemble paper figures
##############
figure1 <- plot_grid(everythingPlot, modelPlot, ncol=2, labels = c("A", "B"))
pdf(paste(outputDir, "figure1.pdf", sep=""), width=15, height=8)
figure1
dev.off()


figureS1 <- plot_grid(confidenceIntPlot, algorithmPlot, goDataSubsetPlot,  organismPlot,  ncol=2, labels = c("A", "B","C", "D"))
pdf(paste(outputDir, "figureS1.pdf", sep=""), width=15, height=15)
figureS1
dev.off()


