library(ggplot2)
library(cowplot)
library(reshape2)
library(readr)
library(ggrepel)
library(readxl)
library(plyr)
library(Hmisc)
inputDir <- "researchBias/inputData/"
outputDir <- "researchBias/"
load(paste(inputDir, "pooledCache1_geneMeta.RData", sep=""))

######
# Functions for identifying particular disease pairs
######
metaMatch <- function(geneDisease, curRankFrame) {
	namePieces <- strsplit(geneDisease, " ")
	geneName <- namePieces[[1]][[1]]
	diseaseName <- gsub(paste(geneName, " ", sep=""), "", geneDisease)
	
	metaRes <- data.frame("effectSize"=t(pooledRes)[,diseaseName], "effectSizeFDR"=t(pooledFDR)[,diseaseName])
	metaRes <- metaRes[!is.na(metaRes$effectSizeFDR),]
	metaRes$esRank <- nrow(metaRes) - rank(abs(metaRes[,1]))
	metaRes$esFDRRank <- rank(metaRes$effectSizeFDR)
	metaRes$geneName <- rownames(metaRes)
	
	metaRow <- metaRes[metaRes$geneName == geneName,]
	if(nrow(metaRow) > 0) {
		metaRow$diseaseName <- diseaseName
		metaRow$geneDisease <- geneDisease
		metaRow$numPubs<- 0
		for(colName in colnames(curRankFrame)) {
			if(!(colName %in% colnames(metaRow))) {
				metaRow[,colName] <- NA
			}
		}
		return(metaRow)
	}
	returnMe <- curRankFrame[1,]
	for(i in 1:ncol(returnMe)) {
		returnMe[,i] <- NA
	}
	return(returnMe)
}

metaComb <- function(geneDisease, curRankFrame) {
	print(geneDisease)
	if(geneDisease %in% curRankFrame$geneDisease) {
		return(curRankFrame[curRankFrame$geneDisease == geneDisease,])
	}
	return(metaMatch(geneDisease, curRankFrame))
}

###########
# Pubpular data
###########
pubpularDir <- paste(inputDir,"pubpular/", sep="")
if(exists("pubpularRankFrame")) {
	rm(pubpularRankFrame)
}
for(filename in list.files(pubpularDir, pattern=".csv")) {
	diseaseName <- strsplit(filename, "\\.")[[1]][[1]]
	
	pubpularRes <- read.csv(paste(pubpularDir, filename, sep=""))
	pubpularRes$countRank <- nrow(pubpularRes) - rank(pubpularRes$Term_count)
	pubpularRes$pRank <- rank(pubpularRes$P)
	pubpularRes$geneName <- pubpularRes$GN
	
	metaRes <- data.frame("effectSize"=t(pooledRes)[,diseaseName], "effectSizeFDR"=t(pooledFDR)[,diseaseName])
	metaRes <- metaRes[!is.na(metaRes$effectSizeFDR),]
	metaRes$esRank <- nrow(metaRes) - rank(abs(metaRes[,1]))
	metaRes$esFDRRank <- rank(metaRes$effectSizeFDR)
	metaRes$geneName <- rownames(metaRes)
	if(length(levels(as.factor(metaRes$esFDRRank))) > 5) {
		mergedRes <- merge(pubpularRes, metaRes, by.x="geneName", by.y="geneName", all.y=T)
	
		tempFrame <- data.frame(diseaseName = diseaseName, esPCor=cor(mergedRes$esRank, mergedRes$pRank, method="spearman"), 
														esCountCor=cor(mergedRes$esRank, mergedRes$countRank, method="spearman"), 
														fdrPCor=cor(mergedRes$esFDRRank, mergedRes$pRank, method="spearman"),
														fdrCountCor=cor(mergedRes$esFDRRank, mergedRes$countRank, method="spearman"))
		mergedRes$diseaseName <- rep(diseaseName, nrow(mergedRes))
		if(exists("pubpularRankFrame")) {
			pubpularRankFrame <- rbind(pubpularRankFrame, mergedRes)
		} else {
			pubpularRankFrame <- mergedRes
		}
	}
}
pubpularRankFrame$geneDisease <- paste(pubpularRankFrame$geneName, pubpularRankFrame$diseaseName)
pubpularRankFrame$numPubs <- pubpularRankFrame$Term_count
pubpularRankFrame$numPubs[is.na(pubpularRankFrame$numPubs)] <- 0

save(pubpularRankFrame, file=paste(inputDir, "pubpularFrames.RData", sep=""))
load(paste(inputDir, "pubpularFrames.RData", sep=""))

metaSubsetVec <- c("PTK7 Adenocarcinoma of Lung", "SMYD2 Pancreatic Cancer", "ZDHHC19 Sepsis", #"C9orf95 Sepsis", 
									 "GNA15 Sepsis", "BATF Sepsis", "C3AR1 Sepsis", "CEACAM1 Sepsis", "KIAA1370 Sepsis", "TGFBI Sepsis", "MTCH1 Sepsis", 
									 "RPGRIP1 Sepsis", #"HLA-DBP1 Sepsis", 
									 "BASP1 Transplant Rejection", "CD6 Transplant Rejection", 
									 "CD7 Transplant Rejection", "CXCL10 Transplant Rejection", "CXCL9 Transplant Rejection", "INPP5D Transplant Rejection",
									 "ISG20 Transplant Rejection", "LCK Transplant Rejection", "NKG7 Transplant Rejection", 
									 "PSMB9 Transplant Rejection", "RUNX3 Transplant Rejection", "TAP1 Transplant Rejection" )
metaSubsetList <- lapply(metaSubsetVec, metaComb, curRankFrame= pubpularRankFrame)
pubpularSubset <- data.frame()
for(i in 1:length(metaSubsetList)) {
	if(i>1) {
		pubpularSubset <- rbind(pubpularSubset, metaSubsetList[[i]][,colnames(pubpularSubset)])
	} else {
		pubpularSubset <- rbind(pubpularSubset, metaSubsetList[[i]])
	}
}
pubpularSubset <- do.call(rbind, metaSubsetList)

### Examine density of pubpular terms
#Calculate % of terms identified at FDR < 5%

pubpularDensityFrame <- pubpularRankFrame[pubpularRankFrame$numPubs > 0,]
fdrDensity <- ecdf(pubpularDensityFrame$effectSizeFDR)
print(fdrDensity(0.05))
pubpularDensityFrame$group <- rep("Top 260", nrow(pubpularDensityFrame))
pubpularDensityFrame <- rbind(pubpularDensityFrame, data.frame(pubpularRankFrame[!is.na(pubpularRankFrame$pRank) & pubpularRankFrame$pRank < 10,], group=rep("Top 10", sum(pubpularRankFrame$pRank < 10, na.rm=T))))
pubpularDensityFrame <- rbind(pubpularDensityFrame, data.frame(pubpularRankFrame[!is.na(pubpularRankFrame$pRank) & pubpularRankFrame$pRank < 5,], group=rep("Top 5", sum(pubpularRankFrame$pRank < 5, na.rm=T))))
pubpularDensityFrame$effectSize <- pubpularDensityFrame$effectSize
pubpularDensityFrame$Pubpular <- factor(pubpularDensityFrame$group, levels=c("Top 5", "Top 10", "Top 260"))

############
# Gene Ontology data
############
library(readr)
geneOntology <- read_tsv(paste(inputDir, "amigoQuery.txt", sep=""))
colnames(geneOntology) <- c("geneOntologyCount", "geneName")
goPubGeneFrame <- merge(pubpularRankFrame, geneOntology, by.x="geneName", by.y="geneName", all.x=TRUE, all.y=FALSE)
metaGO <- merge(pubpularSubset, geneOntology, by.x="geneName", by.y="geneName", all.x=TRUE, all.y=FALSE)
metaGO <- metaGO[!is.na(metaGO$geneName),]

pubpularLabelSubset <- subset(goPubGeneFrame, numPubs>=1000)
pubpularLabelSubset$geneDisease <- paste(pubpularLabelSubset$geneName, pubpularLabelSubset$diseaseName)

###########
# Pubpular plots
###########
figure2A <- ggplot(goPubGeneFrame, aes(x=esFDRRank, y=numPubs)) + geom_hex() + 
	scale_fill_continuous(low = "gray95", high = "black", trans = "log", breaks=c(1,1e2,1e4)) +
	xlab("Rank of Gene Expression in Disease") + ylab("Number of Published Disease Gene Associations") +
	scale_x_continuous(limits=c(NA,10000)) + scale_y_sqrt(breaks=c(0,1,10,100,1e3)) + 
	geom_text_repel(data=pubpularLabelSubset, aes(x=esFDRRank, y=numPubs, label=geneDisease), color="purple", hjust=-0.03, size=7) +
	geom_point(data=pubpularLabelSubset, aes(x=esFDRRank, y=numPubs, label=geneDisease), color="purple") +
	geom_text_repel(data=pubpularSubset[pubpularSubset$geneName %in% c("ZDHHC19", "CXCL9", "PTK7"),], aes(x=esFDRRank, y=numPubs, label=geneDisease), color="orange", hjust=-0.03, size=7) + 
	geom_point(data=pubpularSubset, aes(x=esFDRRank, y=numPubs), color="orange") + guides(color=FALSE) +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F)) 

pubpularDensityPlot <- ggplot(pubpularDensityFrame, aes(x=effectSizeFDR, color=Pubpular)) + stat_ecdf(lwd=2) + 
	xlab("Meta-Analysis Effect Size FDR") + 
	ylab("Cumulative Density") + geom_abline(intercept = 0, slope=1, lwd=2, lty=2)  + geom_vline(xintercept=0.05, lwd=2, lty=2)  + 
	scale_color_manual(values=c("#2b8d00","#0254f1","#99dcb1", "#000000")) + 
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.1,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F))

figure2B <- ggplot(goPubGeneFrame, aes(y=numPubs, x=geneOntologyCount))  + geom_hex() + scale_x_log10()  + 
	scale_y_sqrt(breaks=c(0,1,10,100,1e3)) + 
	scale_fill_continuous(low = "grey95", high = "black", trans = "log", breaks=c(1,1e2,1e4)) +
	xlab("Gene Ontology Annotations (non-IEA)") + ylab("Number of Published Disease-Gene Associations") +
	geom_text_repel(data=pubpularLabelSubset, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="purple", hjust=-0.03, size=7) + 
	geom_point(data=pubpularLabelSubset, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="purple") + 
	geom_text_repel(data=metaGO[metaGO$geneName %in% c("ZDHHC19", "CXCL9", "PTK7"),], aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="orange", hjust=-0.03, size=7) +
	geom_point(data=metaGO, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="orange") + 
	guides(color=FALSE) +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F))

goDiseasePlot <- ggplot(goPubGeneFrame, aes(y=geneOntologyCount, x=esFDRRank))  + geom_hex()+
	scale_fill_continuous(low = "gray95", high = "black", trans = "log", breaks=c(1,1e2,1e4))+ scale_y_log10() + 
	xlab("Rank of Gene Expression in Disease") + ylab("Gene Ontology Annotations (non-IEA)") + scale_x_continuous(limits=c(NA,10000)) +
	geom_text_repel(data=pubpularLabelSubset, aes(x=esFDRRank, y=geneOntologyCount, label=geneDisease), color="purple", hjust=-0.03, size=7) +
	geom_point(data=pubpularLabelSubset, aes(x=esFDRRank, y=geneOntologyCount, label=geneDisease), color="purple") +
	geom_text_repel(data=metaGO[metaGO$geneName %in% c("ZDHHC19", "CXCL10", "PTK7"),], aes(x=esFDRRank, y=geneOntologyCount, label=geneDisease), color="orange", hjust=-0.03, size=7) + 
	geom_point(data=metaGO, aes(x=esFDRRank, y=geneOntologyCount), color="orange") + guides(color=FALSE) + 
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(1,0.05), 
				legend.justification = c(1,0), legend.background = element_rect(fill=alpha("grey",0.5)))

goPubGeneFrameOnePub <- goPubGeneFrame[goPubGeneFrame$numPubs>=10,]
cor.test(goPubGeneFrameOnePub$esFDRRank, goPubGeneFrameOnePub$geneOntologyCount, method="spearman")
cor.test(goPubGeneFrameOnePub$esFDRRank, goPubGeneFrameOnePub$numPubs, method="spearman")
cor.test(goPubGeneFrameOnePub$numPubs, goPubGeneFrameOnePub$geneOntologyCount, method="spearman")

###########
# HUGE Analysis
###########
hugeDir <-  paste(inputDir,"huge/", sep="")

if(exists("hugeFrame")) {
	rm(hugeFrame)
}
for(filename in c(list.files(hugeDir, pattern = "t[xs][tv]"), list.files(hugeDir, pattern = "xlsx"))) {
	diseaseName <- strsplit(strsplit(filename, "\\.")[[1]][[1]], "_")[[1]][[1]]
	print(diseaseName)
	
	if(length(grep("xlsx", filename))>0) {
		diseaseFrame <- read_excel(paste(hugeDir, filename, sep=""), skip=1)
		colnames(diseaseFrame) <- c("geneName", "numPubs", "numMetaPubs")
		diseaseFrame <- diseaseFrame[,c(1,2)]
	} else {
		diseaseFrame <- read_tsv(paste(hugeDir,filename, sep=""), skip=5)
		colnames(diseaseFrame) <- c("geneName", "numPubs")
	}
	diseaseFrame$diseaseName <- rep(diseaseName, nrow(diseaseFrame))
	
	metaRes <- data.frame("effectSize"=t(pooledRes)[,diseaseName], "effectSizeFDR"=t(pooledFDR)[,diseaseName])
	metaRes <- metaRes[!is.na(metaRes$effectSizeFDR),]
	metaRes$esRank <- nrow(metaRes) - rank(abs(metaRes[,1]))
	metaRes$esFDRRank <- rank(metaRes$effectSizeFDR)
	metaRes$geneName <- rownames(metaRes)
	
	mergedRes <- merge(diseaseFrame, metaRes, by="geneName", all.y=T)
	
	if(exists("hugeFrame")) {
		hugeFrame <- rbind(hugeFrame, mergedRes)
	} else {
		hugeFrame <- mergedRes
	}
}
hugeFrame$geneDisease <- paste(hugeFrame$geneName, hugeFrame$diseaseName)
hugeFrame$numPubs[is.na(hugeFrame$numPubs)] <- 0

hugeSubsetList <- lapply(metaSubsetVec, metaComb, curRankFrame=hugeFrame)
hugeSubset <- do.call(rbind, hugeSubsetList)


goHugeFrame <- merge(hugeFrame, geneOntology, by="geneName", all.x=TRUE, all.y=FALSE)
goHugeSubset <- merge(hugeSubset, geneOntology, by="geneName", all.x=TRUE, all.y=FALSE)

hugeLabelSubset <- goHugeFrame[goHugeFrame$numPubs > 500,]

hugeFigure2A <- ggplot(goHugeFrame, aes(x=esFDRRank, y=numPubs)) + geom_hex() + 
	scale_fill_continuous(low = "gray95", high = "black", trans = "log", breaks=c(1,10,1e2,1e3)) +
	xlab("Rank of Gene Expression in Disease") + ylab("Number of Published Disease Gene Associations (HUGE)") +
	scale_x_continuous(limits=c(NA,10000)) + scale_y_sqrt(breaks=c(0,1,10,100,1e3)) + 
	geom_text_repel(data=hugeLabelSubset, aes(x=esFDRRank, y=numPubs, label=geneDisease), color="purple", hjust=-0.03, size=7) +
	geom_point(data=hugeLabelSubset, aes(x=esFDRRank, y=numPubs, label=geneDisease), color="purple") +
	geom_text_repel(data=goHugeSubset[goHugeSubset$geneName %in% c("ZDHHC19", "CXCL9", "PTK7"),], aes(x=esFDRRank, y=numPubs, label=geneDisease), color="orange", hjust=-0.03, size=7) + 
	geom_point(data=goHugeSubset, aes(x=esFDRRank, y=numPubs), color="orange") + guides(color=FALSE) +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F)) 

hugeFigure2B <- ggplot(goHugeFrame, aes(y=numPubs, x=geneOntologyCount))  + geom_hex() + scale_x_log10()  + 
	scale_y_sqrt(breaks=c(0,1,10,100,1e3)) + 
	scale_fill_continuous(low = "grey95", high = "black", trans = "log", breaks=c(1,10,1e2,1e3)) +
	xlab("Gene Ontology Annotations (non-IEA)") + ylab("Number of Published Disease Gene Associations (HUGE)") +
	geom_text_repel(data=hugeLabelSubset, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="purple", hjust=-0.03, size=7) + 
	geom_point(data=hugeLabelSubset, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="purple") + 
	geom_text_repel(data=goHugeSubset[goHugeSubset$geneName %in% c("ZDHHC19", "CXCL9", "PTK7"),], aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="orange", hjust=-0.03, size=7) +
	geom_point(data=goHugeSubset, aes(x=geneOntologyCount, y=numPubs, label=geneDisease), color="orange", hjust=-0.03, size=1) + 
	guides(color=FALSE) +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F))

goHugeFrameOnePub <- goHugeFrame[goHugeFrame$numPubs>=10,]
cor.test(goHugeFrameOnePub$esFDRRank, goHugeFrameOnePub$numPubs, method="spearman")
cor.test(goHugeFrameOnePub$numPubs, goHugeFrameOnePub$geneOntologyCount, method="spearman")

##########
# GWAS catalog data
##########
diseaseNames <- unique(pubpularRankFrame$diseaseName)
gwasData <- read_tsv(paste(inputDir,"gwas_catalog_v1.0.1-associations_e85_r2016-08-25.tsv", sep=""))

gwasSubset <- gwasData[tolower(gwasData$`DISEASE/TRAIT`) %in% tolower(diseaseNames),]
gwasSubset <- gwasData[tolower(gwasData$MAPPED_TRAIT) %in% tolower(diseaseNames),]
# Map disease names to MetaSignature diseases
mappingFunction <- function(curVal) {
	if(curVal == "body mass index") { return("obesity") }
	if(curVal == "type ii diabetes mellitus") { return("type 2 diabetes") }
	if(curVal == "prostate carcinoma") { return("prostate cancer") }
	if(curVal == "waist-hip ratio, body mass index") { return("obesity") }
	if(curVal == "breast carcinoma") { return("breast cancer") }
	if(curVal == "sporadic amyotrophic lateral sclerosis") { return("amyotrophic lateral sclerosis") }
	if(curVal == "type i diabetes mellitus") { return("type 1 diabetes") }
	if(curVal == "hiv-1 infection") { return("human immunodeficiency virus") }
	if(curVal == "pancreatic carcinoma") { return("pancreatic cancer") }
	if(curVal == "iga glomerulonephritis") { return("glomerulonephritis") }
	if(curVal == "chronic lymphocytic leukemia") { return("b-cell chronic lymphocytic leukemia") }
	return(curVal)
}

gwasData$metaDisease <- sapply(tolower(gwasData$MAPPED_TRAIT), mappingFunction)
gwasSubset <- gwasData[tolower(gwasData$metaDisease) %in% tolower(diseaseNames),]
gwasSubset$geneDisease <- paste(gwasSubset$MAPPED_GENE, gwasSubset$metaDisease)
gwasSubset$logOdds <- log2(gwasSubset$`OR or BETA`)
gwasSubset$pVal <- gwasSubset$`P-VALUE`


##########
# Huge SNP data
##########
hugeSNPDir <- paste(inputDir, "hugeSnpData/", sep="")

if(exists("hugeSNP")) {
	rm(hugeSNP)
}
for(filename in list.files(hugeSNPDir, pattern = "t[xs][tv]")) {
	diseaseName <- strsplit(strsplit(filename, "\\.")[[1]][[1]], "_")[[1]][[1]]
	print(diseaseName)
	
	diseaseFrame <- read_tsv(paste(hugeSNPDir,filename, sep=""), skip=4)
	diseaseFrame <- diseaseFrame[diseaseFrame$Gene != "NR",]
	if(nrow(diseaseFrame) > 0) {
		
		diseaseFrame$diseaseName <- rep(diseaseName, nrow(diseaseFrame))
		oddsRatioFun <- function(curStr) {as.numeric(strsplit(curStr, "\\[")[[1]][[1]])}
		diseaseFrame$oddsRatio <- sapply(diseaseFrame$`OR/Beta [95% CI]`, oddsRatioFun)
		
		pValFun <- function(curStr) {as.numeric(gsub("x10","e",curStr))}
		diseaseFrame$pVal <- sapply(diseaseFrame$`p-Value`, pValFun)
		
		splitFun <- function(curRow) { 
			geneNames <- strsplit(curRow["Gene"], ",")[[1]]
			geneNameFun <- function(geneName) {
				tempFrame <- curRow
				tempFrame$geneName <- geneName
				return(tempFrame)
			}
			return(do.call("rbind",lapply(geneNames, geneNameFun)))
		}
		
		diseaseFrame <- do.call("rbind",apply(diseaseFrame, 1, splitFun))
		diseaseFrame <- as.data.frame(diseaseFrame)
		diseaseFrame$geneName <- as.character(diseaseFrame$geneName)
		diseaseFrame$pVal <- as.numeric(diseaseFrame$pVal)
		diseaseFrame$oddsRatio <- as.numeric(diseaseFrame$oddsRatio)
		
		metaRes <- data.frame("effectSize"=t(pooledRes)[,diseaseName], "effectSizeFDR"=t(pooledFDR)[,diseaseName])
		metaRes <- metaRes[!is.na(metaRes$effectSizeFDR),]
		metaRes$esRank <- nrow(metaRes) - rank(abs(metaRes[,1]))
		metaRes$esFDRRank <- rank(metaRes$effectSizeFDR)
		metaRes$geneName <- rownames(metaRes)
		
		mergedRes <- merge(diseaseFrame, metaRes, by="geneName")
		
		if(exists("hugeSNP")) {
			hugeSNP <- rbind(hugeSNP, mergedRes)
		} else {
			hugeSNP <- mergedRes
		}
	}
}
hugeSNP$geneDisease <- paste(as.character(hugeSNP$geneName), tolower(as.character(hugeSNP$diseaseName)))
hugeSNP$logOdds <- log2(hugeSNP$oddsRatio)

#############
# Merge SNPs with Pubpular and Meta
#############
dedupFun <-  function(m) { return(data.frame(logOdds=median(m$logOdds, na.rm=T), pVal=median(m$pVal, na.rm=T))) } 

gwasDedup <- ddply(gwasSubset, .(geneDisease), dedupFun)
hugeDedup <- ddply(hugeSNP, .(geneDisease), dedupFun)

tempRank <- pubpularRankFrame
tempRank$geneDisease <- paste(tempRank$geneName, tolower(tempRank$diseaseName))

gwasRank <- merge(gwasDedup, tempRank, by="geneDisease", all.x=T)
hugeSnpRank <- merge(hugeDedup, tempRank, by="geneDisease", all.x=T)
gwasRank$numPubs[is.na(gwasRank$numPubs)] <- 0
hugeSnpRank$numPubs[is.na(hugeSnpRank$numPubs)] <- 0

gwasLabel <- gwasRank[gwasRank$numPubs >=1000,]
hugeSnpLabel <- hugeSnpRank[hugeSnpRank$numPubs >=1000,]

modRank <- gwasRank
modRank$pVal[modRank$pVal < 1e-40] <- 1e-40

gwasPubFigure <- ggplot(gwasRank, aes(x=pVal, y=numPubs))+ xlab("GWAS Catalog SNP p-value")+ geom_hex() + 
	scale_fill_continuous(low = "gray95", high = "black", trans = "log", breaks=c(1,10, 1e2, 1e3)) + 
	scale_y_sqrt(breaks=c(0,10,100,1e3)) + 
	ylab("Number of Published Disease-Gene Associations") + 
	scale_x_log10(breaks=c(1e-100, 1e-50, 1e-20, 1))+
	geom_text_repel(data=gwasLabel,aes(label=geneDisease),  color="purple",hjust=-0.03, size=7)  + geom_point(data=gwasLabel,  color="purple") +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F)) 
cor.test(gwasRank[gwasRank$numPubs>=10,]$pVal, gwasRank[gwasRank$numPubs>=10,]$numPubs, method="spearman")

hugeSnpFigure <- ggplot(hugeSnpRank, aes(x=pVal, y=numPubs)) + xlab("HUGE SNP p-value")+ geom_hex() + 
	scale_fill_continuous(low = "gray95", high = "black", trans = "log", breaks=c(1,10,1e2,1e3)) + scale_x_log10(breaks=c(1e-150, 1e-100, 1e-50, 1))+
	scale_y_sqrt(breaks=c(0,1,10,100,1e3)) +ylab("Number of Published Disease-Gene Associations") + 
	geom_text_repel(data=hugeSnpLabel,aes(label=geneDisease),  color="purple",hjust=-0.03, size=7)  + geom_point(data=hugeSnpLabel,  color="purple") +
	theme(text = element_text(size=20), axis.title=element_text(size=24),  legend.position = c(0.05,0.95), 
				legend.justification = c(0,1), legend.background = element_rect(fill=F)) 
cor.test(hugeSnpRank[hugeSnpRank$numPubs>=10,]$pVal, hugeSnpRank[hugeSnpRank$numPubs>=10,]$numPubs, method="spearman")


###########
# Assemble figures
###########

figure2 <- plot_grid(figure2A, figure2B, ncol=2, labels = capitalize(letters[1:2]))
pdf(paste(outputDir, "figure2.pdf", sep=""), width=15, height=8)
figure2
dev.off()

figureS2 <- plot_grid(pubpularDensityPlot, gwasPubFigure,goDiseasePlot, hugeFigure2A, hugeFigure2B, hugeSnpFigure, ncol=2, labels = capitalize(letters[1:6]))
pdf(paste(outputDir, "figureS2.pdf", sep=""), width=20, height=30)
figureS2
dev.off()